@@ -1,8 +1,6 @@
-__version__ = "0.0.1"
+__version__ = "0.0.5"
 
-
-
-_POPYROUS_IMPORT_EVERYTHING_ELSE_ = False
+_POPYROUS_IMPORT_EVERYTHING_ELSE_ = True
 
 if _POPYROUS_IMPORT_EVERYTHING_ELSE_:
     from . matlab import *
@@ -11,3 +9,4 @@ if _POPYROUS_IMPORT_EVERYTHING_ELSE_:
     from . zipfiles import *
     from . timeseries import *
     from . utils import *
+    from . ml import *
