@@ -11,17 +11,16 @@ from sklearn.preprocessing import StandardScaler, MinMaxScaler
 
 
 class TabularDataset:
-    DTYPE = np.float32
-    POWERS_OF_TWO = True
     def __init__(self, 
-        input_vec, in_seq_length:int, in_features:int, in_squeezed:bool,
-        output_vec=None, out_seq_length:int=1, out_features:int=0, out_squeezed:bool=True,
+        input_vec, in_seq_length:int, in_squeezed:bool,
+        output_vec=None, out_seq_length:int=1, out_squeezed:bool=True,
         data_downsampling_rate:int=1, sequence_downsampling_rate:int=1, 
         input_scaling:bool=True, output_scaling:bool=True,
         input_forward_facing:bool=True, output_forward_facing:bool=True,
         input_include_current_timestep:bool=True, output_include_current_timestep:bool=True,
         input_towards_future:bool=False, output_towards_future:bool=True,
-        stacked=True, extern_input_scaler=None, extern_output_scaler=None, scaling:str='standard', verbose=True):
+        stacked=True, extern_input_scaler=None, extern_output_scaler=None, scaling:str='standard', 
+        dtype=np.float32, powers_of_two:bool=True, verbose=True):
         """Generate 3D-tabulated dataset using sliding window from multi-variate timeseries data to be used later in 
         timeseries classification,. regression, deep learning, PyTorch dataloaders, etc.
 
@@ -29,14 +28,12 @@ class TabularDataset:
 
         :param `input_vec` (numpy array):               [Nx1] vector of timeseries inputs.
         :param `in_seq_length` (int):                   Input sequence length (sliding time window length)
-        :param `in_features` (int)                      Number of features of input
         :param `in_squeezed` (bool):                    Squeezed inputs (batch, seqlen*features) as in ANNs 
                                                         or unsqueezed (batch, seqlen, features) as in LSTM
         :param `output_vec` (numpy array):              [Nx1] vector of timeseries outputs (targets). 
                                                         Default is the same as inputs (for autoregression).
         :param `out_seq_length` (int):                  Output (target) sequence length (sliding time window length).
                                                         Default is 1 (estimation rather than forecasting)
-        :param `out_features` (int):                    Number of features of the output target. Default is 0.
         :param `out_squeezed` (bool):                   Squeezed inputs (batch, seqlen*features) as in Dense outputs
                                                         or unsqueezed (batch, seqlen, features) as in RNN outputs
         :param `data_downsampling_rate` (int):          Downsampling rate, if the data has too high sampling rate.
@@ -71,6 +68,11 @@ class TabularDataset:
         :param `extern_output_scaler` (sklearn scaler): If not None, this scaler will be used to scale the outputs.
                                                         Default is None.
         :param `scaling` (str):                         Scaling method to use. Default is 'standard'. Other is 'minmax'.
+        :param `dtype` (type):                          Data type to convert all inputs and outputs. Defautls to `np.float32`.
+        :param `powers_of_two` (bool):                  Whether to round final sequence lengths to the nearest power of two. Defaults to True.
+                                                        It is recommended to keep this option True, since numerical computing on GPU is more efficient
+                                                        with arrays whose size are powers of two.
+        :param `verbose` (bool):                        Verbosity. Defaults to True.
 
 
         ### Attributes:
@@ -93,30 +95,25 @@ class TabularDataset:
         self.`_in_seq_length_ds` (int): Input sequence length (sliding time window length) after downsampling
         self.`_out_seq_length` (int): Output sequence length (sliding time window length)
         self.`_out_seq_length_ds` (int): Output sequence length (sliding time window length) after downsampling
-        self.`_in_features` (int):Number of features of input
-        self.`_out_features` (int):Number of features of input
         """
-
+        self.dtype = dtype
+        self.powers_of_two = powers_of_two
+        
         # Make output:
         if output_vec is None:
-            output_vec = input_vec.astype(self.DTYPE)
+            output_vec = input_vec.astype(self.dtype)
         
         # Reshape inputs and outputs
         if len(input_vec.shape)==1:
-            input_vec=input_vec.reshape(-1,1).astype(self.DTYPE)
+            input_vec=input_vec.reshape(-1,1).astype(self.dtype)
         if len(output_vec.shape)==1:
-            output_vec=output_vec.reshape(-1,1).astype(self.DTYPE)
-
-        if out_features==0:
-            out_features = output_vec.shape[-1]
+            output_vec=output_vec.reshape(-1,1).astype(self.dtype)
 
         # Reassure size of data.
-        assert len(input_vec.shape)==2 and input_vec.shape[1]==in_features, \
-            "All inputs and outputs must be NumData x NumFeatures vectors. "+\
-            "Inputs and Outputs must have the same number of rows (datapoints)."
-        assert len(output_vec.shape)==2 and output_vec.shape[1]==out_features, \
-            "All inputs and outputs must be NumData x NumFeatures vectors. "+\
-            "Inputs and Outputs must have the same number of rows (datapoints)."
+        assert len(input_vec.shape)==2, \
+            "All inputs and outputs must be NumData x NumFeatures vectors. "
+        assert len(output_vec.shape)==2, \
+            "All inputs and outputs must be NumData x NumFeatures vectors. "
 
         # Reassure inputs and outputs have the same size.
         assert input_vec.shape[0]==output_vec.shape[0], \
@@ -136,8 +133,6 @@ class TabularDataset:
         self._out_seq_length = out_seq_length
         self._input_is_sequence = (in_seq_length > 1)
         self._output_is_sequence = (out_seq_length > 1)
-        self.in_features = in_features
-        self.out_features = out_features
         self.data_downsampling_rate = data_downsampling_rate
         self.sequence_downsampling_rate = sequence_downsampling_rate
 
@@ -156,8 +151,6 @@ class TabularDataset:
         else:
             self._in_seq_length_ds = in_seq_length
             self._out_seq_length_ds = out_seq_length
-        
-
 
         # Scaling inputs and outputs
         if input_scaling:
@@ -166,7 +159,7 @@ class TabularDataset:
             else:
                 inscaler = StandardScaler() if 'standard' in scaling.lower() else MinMaxScaler()
                 inscaler.fit(input_vec)
-            input_vec = inscaler.transform(input_vec).astype(self.DTYPE)
+            input_vec = inscaler.transform(input_vec).astype(self.dtype)
         else:
             inscaler = None
 
@@ -176,7 +169,7 @@ class TabularDataset:
             else:
                 outscaler = StandardScaler() if 'standard' in scaling.lower() else MinMaxScaler()
                 outscaler.fit(output_vec)
-            output_vec = outscaler.transform(output_vec).astype(self.DTYPE)
+            output_vec = outscaler.transform(output_vec).astype(self.dtype)
         else:
             outscaler = None
         
@@ -191,7 +184,7 @@ class TabularDataset:
             inputObj = sliding_window(input_vec, self._in_seq_length_ds, self.sequence_downsampling_rate, 
                 forward_facing=input_forward_facing, 
                 include_current_step=input_include_current_timestep, squeezed=self.in_squeezed, stacked=self.stacked,
-                includes_future_data=input_towards_future, dtype=self.DTYPE)
+                includes_future_data=input_towards_future, dtype=self.dtype, powers_of_two=self.powers_of_two)
             table_in = inputObj["table"]
             self._in_seq_length_final = inputObj["seq_len_ds"]
         else:
@@ -204,7 +197,7 @@ class TabularDataset:
             outputObj = sliding_window(output_vec, self._out_seq_length_ds, self.sequence_downsampling_rate,
                 forward_facing=output_forward_facing, 
                 include_current_step=output_include_current_timestep, squeezed=self.out_squeezed, stacked=self.stacked,
-                includes_future_data=output_towards_future, dtype=self.DTYPE)
+                includes_future_data=output_towards_future, dtype=self.dtype, powers_of_two=self.powers_of_two)
             table_out = outputObj["table"]
             self._out_seq_length_final = outputObj["seq_len_ds"]
         else:
@@ -212,10 +205,10 @@ class TabularDataset:
             self._out_seq_length_final = 1
         if verbose: print("Output Table Shape: ", table_out.shape)
 
-        self.table_in = table_in.astype(self.DTYPE)
-        self.table_out = table_out.astype(self.DTYPE)
+        self.table_in = table_in.astype(self.dtype)
+        self.table_out = table_out.astype(self.dtype)
         self.shape = {"in":self.table_in.shape, "out":self.table_out.shape}
-        if verbose: print("Dataset constructed successfully.")
+        if verbose: print("TabularDataset constructed successfully.")
     
 
 
@@ -295,8 +288,8 @@ def make_squeezed_dataset(hparams:dict, inputs, outputs, **kwargs):
 
     # Generating dataset
     dataset = TabularDataset(
-        inputs, in_sequence_length, in_features=hparams['in_features'], in_squeezed=True, 
-        output_vec=outputs, out_seq_length=out_sequence_length, out_features=hparams['out_features'], 
+        inputs, in_sequence_length, in_squeezed=True, 
+        output_vec=outputs, out_seq_length=out_sequence_length, 
         out_squeezed=True, data_downsampling_rate=data_downsampling, 
         sequence_downsampling_rate=seq_downsampling, 
         **kwargs)
@@ -364,8 +357,8 @@ def make_unsqueezed_dataset(hparams:dict, inputs, outputs, **kwargs):
     # print("Output sequence length:", out_sequence_length)
     
     dataset = TabularDataset(
-        input_vec=inputs, in_seq_length=in_sequence_length, in_features=hparams['in_features'], in_squeezed=False,
-        output_vec=outputs, out_seq_length=out_sequence_length, out_features=hparams['out_features'], 
+        input_vec=inputs, in_seq_length=in_sequence_length, in_squeezed=False,
+        output_vec=outputs, out_seq_length=out_sequence_length, 
         out_squeezed=True, data_downsampling_rate=data_downsampling, sequence_downsampling_rate=seq_downsampling,
         **kwargs)
 
